from utils.read_files import read_paths

def build_prompt(params):
    prompt = "Generate API documentation with table of contents, models list, JSON and curl examples of them in Swagger style using MarkDown based in these models:\n"
    
    print("Accessing models paths...")
    prompt = prompt + read_paths(params["model_paths"], params["verbose"])

    prompt = prompt + "\n And these handlers: \n"
    
    print("\nAccessing api paths...")
    prompt = prompt + read_paths(params["api_paths"], params["verbose"])

    if params["verbose"]:
        print("\nPrompt content:")
        print(prompt)

    return [{"role": "user", "content": prompt}]


import argparse

def cli_help_interface():
    parser=argparse.ArgumentParser(
        prog='genDocsGPT',
        description='''Script to generate API documentation in MarkDown based in the models and api handlers for given project using ChatGPT.''',
        epilog="""We do not recommend using it in non-open-source projects.""")
    parser.add_argument('-v, --verbose', metavar='', help='Show all the messages to debug')
    parser.add_argument('-m / --model', metavar='<model_paths>', required=True, type=str, help='Model paths. If more than one, separate with a comma')
    parser.add_argument('-a / --api', metavar='<api_paths>', required=True, type=str, help='API paths. If more than one, separate with a comma')
    parser.add_argument('-o / --output', metavar='<filename>',type=str, default="./api.md", help='Filename to the documentation generated. Default: ./api.md')
    parser.add_argument('-t / --token', metavar='<token>',type=str, help='ChatGPT API KEY')
    parser.add_argument('--env', metavar='<filename>',type=str, default="./.env", help='Env file that MUST and ONLY have API_KEY="chatgpt_api_key". Default: ./.env')
    parser.parse_args()


import os
import pytest

from utils.get_token import get_token

def setup():
    os.mkdir("./utils/tests/fixtures")
    file = open("./utils/tests/fixtures/.env", "w")
    file.write("API_KEY='token'")
    file.close()

@pytest.fixture
def token():
    return {
        "token": "key",
    }

@pytest.fixture
def params():
    return {
        "env_file": "./utils/tests/fixtures/.env",
        "token": ""
    }

def teardown():
    os.remove("./utils/tests/fixtures/.env")
    os.rmdir("./utils/tests/fixtures")

def test_get_token(token):
    result = get_token(token)
    
    assert result == "key"

def test_get_token_file(params):
    result = get_token(params)
    
    assert result == "token"


import os
import glob

def read_file(file, verbose):
    if verbose:
        print(file)
            
    if os.path.isfile(file):
        text_file = open(file, "r")

        text = text_file.read()

        text_file.close()

        return text
    
    return ""

def read_paths(paths, verbose):
    print("Reading files...")

    str_files = ""
    for path in paths:
        for file in glob.iglob(path + '/**', recursive=True):
            if str_files == "":
                str_files = read_file(file, verbose)
            else:
                str_files = str_files + os.linesep + read_file(file, verbose)
    
    return str_files


import os
import pytest

from utils.write_doc import write_documentation

@pytest.fixture
def params():
    return {
        "output_file": "./utils/tests/filename.md"
    }

@pytest.fixture
def response():
    return "Testing..."

def teardown():
    os.remove("./utils/tests/filename.md")

def test_write_documentation(params, response):
    write_documentation(params, response)

    final_text = response + "\n\n*This documentation file was generated using [genDocsGPT](https://github.com/marco-rosner/genDocsGPT)*"

    file = open("./utils/tests/filename.md", "r")
    text = file.read()
    file.close()

    assert final_text == text 


import openai

def generate_docs_with_chatgpt(prompt, token, model="gpt-3.5-turbo"):
    print("\nGenerating API docs...")

    openai.api_key = token

    response = openai.ChatCompletion.create(
        model=model,
        messages=prompt,
        n=1,
        stop=None,
        temperature=0.5,
    )

    message = response.choices[0].message.content.strip()

    return message


import pytest

from utils.check_params import check_params

@pytest.fixture
def args():
    return ['genDocsGPT.py', '-m', '../lightweight-go-server/models', '-a', '../lightweight-go-server/gin']

@pytest.fixture
def args_t():
    return ['genDocsGPT.py', '-v', '-m', '../lightweight-go-server/models', '-a', '../lightweight-go-server/gin', '-t', 'token']

@pytest.fixture
def args_e():
    return ['genDocsGPT.py', '-v', '-m', '../lightweight-go-server/models', '-a', '../lightweight-go-server/gin', '--env', './.genDocsGPT']

@pytest.fixture
def args_o():
    return ['genDocsGPT.py', '-v', '-m', '../lightweight-go-server/models', '-a', '../lightweight-go-server/gin', '-o', 'filename.md']

@pytest.fixture
def args_verbose():
    return ['genDocsGPT.py', '--verbose', '--model', '../lightweight-go-server/models', '--api', '../lightweight-go-server/gin', '--output', 'filename.md']

@pytest.fixture
def args_f():
    return ['genDocsGPT.py', '-X']

def test_check_params(args):
    result = check_params(args)

    assert result["model_paths"] == ["../lightweight-go-server/models"]
    assert result["api_paths"] == ["../lightweight-go-server/gin"]
    assert result["output_file"] == "./api.md"
    assert result["env_file"] == "./.env"
    assert result["token"] == ""
    assert result["verbose"] == False

def test_check_params_token(args_t):
    result = check_params(args_t)

    assert result["model_paths"] == ["../lightweight-go-server/models"]
    assert result["api_paths"] == ["../lightweight-go-server/gin"]
    assert result["output_file"] == "./api.md"
    assert result["token"] == "token"
    assert result["verbose"] == True

def test_check_params_env_file(args_e):
    result = check_params(args_e)

    assert result["model_paths"] == ["../lightweight-go-server/models"]
    assert result["api_paths"] == ["../lightweight-go-server/gin"]
    assert result["output_file"] == "./api.md"
    assert result["env_file"] == "./.genDocsGPT"
    assert result["token"] == ""
    assert result["verbose"] == True

def test_check_params_output_file(args_o):
    result = check_params(args_o)

    assert result["model_paths"] == ["../lightweight-go-server/models"]
    assert result["api_paths"] == ["../lightweight-go-server/gin"]
    assert result["output_file"] == "filename.md"
    assert result["env_file"] == "./.env"
    assert result["token"] == ""
    assert result["verbose"] == True

def test_check_params_verbose(args_verbose):
    result = check_params(args_verbose)

    assert result["model_paths"] == ["../lightweight-go-server/models"]
    assert result["api_paths"] == ["../lightweight-go-server/gin"]
    assert result["output_file"] == "filename.md"
    assert result["env_file"] == "./.env"
    assert result["token"] == ""
    assert result["verbose"] == True

def test_check_params_fail(args_f):
    with pytest.raises(SystemExit) as pytest_wrapped_e: check_params(args_f)
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 2


def write_documentation(params, response):
    f = open(params["output_file"], "w")

    f.write(response)

    f.write("\n\n")
    f.write("*This documentation file was generated using [genDocsGPT](https://github.com/marco-rosner/genDocsGPT)*")
    
    f.close()

    print("API documentation generated")


import os
import sys

def get_token(params):
    if params["token"] != "":
        return params["token"]

    if os.path.isfile(params["env_file"]):
        text_file = open(params["env_file"], "r")

        text = text_file.read()

        text_file.close()

        return text[9:-1]
    
    print("Make sure that the {0} file exist and has only API_KEY='chatgpt_api_key' in it.".format(params["env_file"]))
    print("For more information about how to configure API_KEY: genDocsGPT -h")
    sys.exit(2)


import sys
import getopt

from utils.help import cli_help_interface

def validate_arg(arg, help):
    if arg=="":
        print("{0} is missing. Try -h or --help to use the command correct.".format(help))
        sys.exit(2)

def check_params(args):
    model_paths = ""
    api_paths = ""
    token = ""
    verbose = False

    try:
        opts, args = getopt.getopt(args[1:], "vhm:a:o:t:", ["verbose", "help", "model=", "api=", "output=", "token=", "env="])
    except:
        cli_help_interface()

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            cli_help_interface()
        elif opt in ("-v", "--verbose"):
            verbose = True
        elif opt in ("-m", "--model"):
            model_paths = arg
        elif opt in ("-a", "--api"):
            api_paths = arg
        elif opt in ("-o", "--output"):
            output_file = arg
        elif opt in ("-t", "--token"):
            token = arg
        elif opt in ("--env"):
            env_file = arg

    try:
        output_file
    except NameError:
        output_file = "./api.md" # default output file

    try:
        env_file
    except NameError:
        env_file = "./.env" # default env file
    
    validate_arg(model_paths, "Model path (-m, --model)")
    validate_arg(api_paths, "Api path (-a, --api)")

    return {
        "model_paths": [model_paths],
        "api_paths": [api_paths],
        "output_file": output_file,
        "token": token,
        "env_file": env_file,
        "verbose": verbose
    }




import os
import pytest

from utils.build_prompt import build_prompt

def setup():
    os.mkdir("./utils/tests/fixtures")
    file = open("./utils/tests/fixtures/fixture.txt", "w")
    file.write("testing...")
    file.close()

@pytest.fixture
def params():
    return {
        "model_paths": ["./utils/tests/fixtures"],
        "api_paths": ["./utils/tests/fixtures"],
        "verbose": False
    }

def teardown():
    os.remove("./utils/tests/fixtures/fixture.txt")
    os.rmdir("./utils/tests/fixtures")

def test_build_prompt(params):
    result = build_prompt(params)

    expected = [{'content': 'Generate API documentation with table of contents, models list, JSON and curl examples of them in Swagger style using MarkDown based in these models:\ntesting...\n And these handlers: \ntesting...', 'role': 'user'}]
    
    assert result == expected


import os
import pytest

from utils.read_files import read_paths, read_file

def setup():
    os.mkdir("./utils/tests/fixtures")
    file = open("./utils/tests/fixtures/fixture.txt", "w")
    file.write("testing...")
    file.close()

@pytest.fixture
def paths():
    return ["./utils/tests/fixtures"]

def teardown():
    os.remove("./utils/tests/fixtures/fixture.txt")
    os.rmdir("./utils/tests/fixtures")

def test_read_paths(paths, verbose=False):
    result = read_paths(paths, verbose)

    file = open(paths[0] + "/fixture.txt", "r")
    text = file.read()
    file.close()

    assert result == text

def test_read_file_with_path(paths, verbose=False):
    result = read_file(paths[0], verbose)

    file = open(paths[0] + "/fixture.txt", "r")
    text = file.read()
    file.close()

    assert result != text

def test_read_file_with_filename(paths, verbose=False):
    result = read_file(paths[0] + "/fixture.txt", verbose)

    file = open(paths[0] + "/fixture.txt", "r")
    text = file.read()
    file.close()

    assert result == text


import sys

from utils.check_params import check_params
from utils.gen_docs import generate_docs_with_chatgpt
from utils.build_prompt import build_prompt
from utils.write_doc import write_documentation
from utils.get_token import get_token

def main():
    params = check_params(sys.argv)
    
    prompt = build_prompt(params)

    token = get_token(params)
    
    chatgpt_response = generate_docs_with_chatgpt(prompt, token)

    write_documentation(params, chatgpt_response)